/* Filename: rs232.c
   Purpose:  Serial Communication Interface
   Compiler: wxDevCpp Version 7.3.1.3 (MINGW32)
   Project:  OutGauge Bus
   Original Author:   Vladimir Kadlec http://www.fi.muni.cz/~xkadlec
   
   Revision: Serial Interface and Packet Update revisions  - Pete Willard
   Version: LFS.Z28.01
             
   Additional Source Code:  Peter Anderson
            
Routines:   
   RS232_init       Supply the COM Port # and set up communications
   RS232_display    SEND Serial data to rcv bus.
   RS232_clear      SEND shutdown command to all listeners

*/

/*

ORIGINAL: Gear Indicator is free software; you can redistribute it and/or 
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Gear Indicator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Gear Indicator; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//
// The goal of this set of routines is to read the outgauge packets and
// generate the appropriate Serial string to be send to the Outgauge Bus.
// Listeners on the serial bus respond to preset commands and provide an
// expandable dashboard bus for Gear indicators, RPM gauges and lights, 
// warnings, etc... 
//
// <start><cmd><value><end><cr><lf>
// !G1*<cr><lf>   Set to gear 1
// 
#include <conio.h>
#include <stdio.h>
#include <windows.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>

#include <unistd.h>

#include <time.h>
#include <math.h>

#include "rs232.h"
#include "log.h"
#include "outgauge.h"
#include "config.h"           // external config file routines



int io_port;


// Idle/Max RPM table provided by Nick Aronson for S2 version Y 
// Hopefully just fine for version Z28
char Cars[20][4]   = {"XFG","XRG","XRT","RB4","FXO","LX4","LX6","MRT","UF1","RAC","FZ5","XFR","UFR","FOX","FO8","BF1","FXR","XRR","FZR","FBM"};
int CarIdleRPM[20] = {1000 , 1000 , 1000 , 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1500, 1500, 2000, 2000, 3500, 1500, 1500, 1500, 2000};
int CarMaxRPM[20]  = {8000, 7000, 7500, 7500, 7500, 9000, 9000, 9000, 7000, 7000, 8000, 8000, 9000, 7500, 9500, 20000,7500, 7500, 8500, 9200};
float in_x;
float in_min; 
float in_max; 
float out_min; 
float out_max;

char s[25];        // Serial data string
char sgrs[3] = "";      
char srpm[3] = "";
char spit[3] = "";
char shft[3] = "";
char slow[3] = "";

HANDLE hCom; 

//----------------------------------------------------------------------------
// RS232 routines

HANDLE rs232_initialise (const long int BaudRate,
                         const char parity, const char data);
void rs232_flush(void);
void rs232_terminate(void);
char rs232_getch(void);
int  rs232_getline(char line[], clock_t timeout);
void rs232_putch(int txchar);
void rs232_putstr(const char *string);

// delay_routines
void delay_ms(clock_t millis);

float map(float in_x, float in_min, float in_max, float out_min, float out_max);


//-----------------------------------------------------------------------------
int RS232_init (int com, int speed) {

//handle for serial port I/O
    io_port = com;

    if(!rs232_initialise(speed, '8', 'N'))    // open the COM port
    {
         error("Opening Port Failed\n");
         return(0);
    }
   return (1);

}

//-----------------------------------------------------------------------------
int RS232_clear (Config *conf) {
    sprintf(s, "C0\n\r");    // Clear Command
    rs232_putstr(s);
    return (1);
}


//-----------------------------------------------------------------------------
int RS232_display (Config *conf, OutGaugePack *packet) {

int  i;

float MaxRpm;
float MinRpm;
float RpmRange; 
int CarNo;
int LedScale;  // MAX 8 due to the fact that only a single byte is desired

word flags;



//-----[ GEAR ]----------------------------------------------------------------
// 0 = reverse  1 = Neutral  2 = gear 1... etc                 

    sprintf(sgrs, "G%d", packet->Gear); 
 

//-----[ RPM ]-----------------------------------------------------------------
// DETERMINE CAR

    for (i = 0; i < 20 ; i++) {
        if (strcasecmp (packet->Car, Cars[i]) == 0) {
           CarNo=i;
           }
    }
    
// DETERMINE RPM BAR GRAPH VALUES BASED ON CAR
//   Create a scale indicator for the upper 2portion of the RPM range
//   based on the Min / Max RPM values defined for each car and by the
//   number of LED's  (1-8) used to indicate RPM. Load an array with the values
//   and then compare against actual RPM.  

    MaxRpm = CarMaxRPM[CarNo];   // Pull the values from Nick's static RPM list
    MinRpm = CarIdleRPM[CarNo];
    // LED's start to turn on at 80% of MAX RPM value
    RpmRange = (MinRpm +(MaxRpm-MinRpm)*0.8  ); 
    LedScale = map(packet->RPM,RpmRange,MaxRpm,1,conf->rpm_leds);
    // Scale Bounding    
    if (LedScale <=0) {
        LedScale = 0;
    }
    sprintf(srpm,"R%d",LedScale);


// PitSpeed
flags = packet->ShowLights;
if ((flags & DL_PITSPEED) > 0) {
          sprintf(spit,"P1");
 
   } else {
          sprintf(spit,"P0");
 
}



// SHIFT LIGHT
//if ((packet->DashLights & DL_SHIFT) > 0){
//    printf (" [ Have Shift a Shift Light! ]"); 
//} 

flags = packet->ShowLights;

    if ((flags & DL_SHIFT) > 0) {
          sprintf(shft,"S1");
    } else {
          sprintf(shft,"S0");          
    }



// LOW FUEL

if (packet->Fuel >= conf->fuel_warning*0.01) {
         sprintf(slow,"F0");
   } else {
         sprintf(slow,"F1");
}  


 
 // Gear, RPM, Pit Limit, Shift Light, Low Fuel
 sprintf(s,"%s,%s,%s,%s,%s\n\r",sgrs,srpm,spit,shft,slow);
 rs232_putstr(s);         
 return (1);
}

//-----------------------------------------------------------------------------
//

void delay_ms(clock_t millis)
{
   clock_t endtime;
   endtime = millis + clock();
   while( endtime > clock() );

}


//-----------------------------------------------------------------------------
// RS232 -- MICROSOFT API routines
//-----------------------------------------------------------------------------
HANDLE rs232_initialise (const long int BaudRate, const char parity, const char data)
{
    BOOL bPortReady;
    DCB dcb;
	COMMTIMEOUTS CommTimeouts;

    char ComPortName[12];
    
    sprintf(ComPortName,"\\\\.\\Com%i",io_port); // Allows USB devices @ COM10+

    
    hCom = CreateFile(ComPortName, GENERIC_READ | GENERIC_WRITE,
                                  0,            // exclusive access
                                  NULL,         // no security
                                  OPEN_EXISTING,
                                  0,            // no overlapped I/O
                                  NULL);        // null template


    if ((int)hCom <= 0)
    {
        // Note: GetLastError() requires (%lu) to be used or else a warning
        //       is thrown.          
        printf("Serial port COM%d connect fail %s error (%lu)\n\r", 
                io_port, ComPortName,  GetLastError());
        return 0;
    }
    //else  printf(" serial port COM%d connect OK \n\r", io_port);

    bPortReady = SetupComm(hCom, 128, 128); // set buffer sizes
    if (!bPortReady )
    {
        printf("Serial port COM%d SetupComm fail  (%lu)\n\r", 
                io_port,   GetLastError());
        return 0;
    }
    //else                printf(" serial port COM%d connect OK \n\r", io_port);

    bPortReady = GetCommState(hCom, &dcb);
    if (!bPortReady )
    {
        printf("Serial port COM%d  GetCommState fail  (%lu)\n\r", 
                io_port,   GetLastError());
        return 0;
    }
    //  else   printf(" serial port COM%d connect OK \n\r", io_port);
    dcb.BaudRate = BaudRate;
    if( data == '7') dcb.ByteSize = 7;
    else             dcb.ByteSize = 8;
    if( parity == 'E') dcb.Parity = EVENPARITY;
    if( parity == 'O') dcb.Parity = ODDPARITY;
    else               dcb.Parity = NOPARITY;
    dcb.StopBits = ONESTOPBIT;
    dcb.fAbortOnError = TRUE;

    // set XON/XOFF
    dcb.fOutX = FALSE;                       // XON/XOFF off for transmit
    dcb.fInX = FALSE;                        // XON/XOFF off for receive
    // set RTSCTS
    dcb.fOutxCtsFlow = FALSE;               // turn off CTS flow control
    dcb.fRtsControl = FALSE;                // RTS_CONTROL_HANDSHAKE; //
    // set DSRDTR
    dcb.fOutxDsrFlow = FALSE;               // turn off DSR flow control
    //dcb.fDtrControl = DTR_CONTROL_ENABLE; // DTR handshake
    dcb.fDtrControl = DTR_CONTROL_DISABLE;  //
    // dcb.fDtrControl = DTR_CONTROL_HANDSHAKE; //

    bPortReady = SetCommState(hCom, &dcb);
    if (!bPortReady )
    {
        printf("Serial port COM%d  SetCommState fail  (%lu)\n\r", 
                io_port,  GetLastError());
        return 0;
    }

    // Communication timeouts
    //COMMTIMEOUTS CommTimeouts;
    bPortReady = GetCommTimeouts (hCom, &CommTimeouts);
    CommTimeouts.ReadIntervalTimeout = 5 ;
    CommTimeouts.ReadTotalTimeoutConstant = 5 ;
    CommTimeouts.ReadTotalTimeoutMultiplier = 1 ;
    CommTimeouts.WriteTotalTimeoutConstant = 5 ;
    CommTimeouts.WriteTotalTimeoutMultiplier = 1 ;
    bPortReady = SetCommTimeouts (hCom, &CommTimeouts);
    if (!bPortReady )
    {
        printf("Serial port COM%d SetCommTimeouts fail  (%lu)\n\r", 
                io_port,  GetLastError());
        return 0;
    }
    else
    {
        printf(" \nSerial port COM%d connect OK \n\r", io_port);
    }
    return (hCom);
}
//-----------------------------------------------------------------------------
void rs232_terminate(void)
{
   CloseHandle(hCom);
}
//-----------------------------------------------------------------------------
char rs232_getch(void)
{
    char rxchar;
    BOOL bReadRC;
    static DWORD iBytesRead;
    bReadRC = ReadFile(hCom, &rxchar, 1, &iBytesRead, NULL);
    if (iBytesRead)
    {
        return rxchar;
    }
    else
    {
        return 0;         // return 0 if no character read
    }
}
//-----------------------------------------------------------------------------
void rs232_flush(void)
{
    while(rs232_getch()!=0)   ;
}
//-----------------------------------------------------------------------------
int rs232_getline(char line[], clock_t timeout)
{
    int num_chars = 0;
    char ch;
    clock_t endtime;
    endtime = timeout + clock();
    //printf("%ld %ld\n", clock(), endtime);
    while(endtime > clock())
    {
        //printf("!");
        ch = rs232_getch();
        //printf("%d ", ch);
        if (ch != 0)
        {
            //printf("%c", ch);
            if ((ch == 10) || (ch == 13))
            {
                line[num_chars] = '\0'; // terminate the string
                return(num_chars);
            }

            else
            {
                line[num_chars] = ch;
                ++num_chars;
            }
        }

    } // end of while
    line[num_chars] = '\0';
    return(-1);  // timeout
}
//-----------------------------------------------------------------------------
void rs232_putch(int txchar)
{
    BOOL bWriteRC;
    static DWORD iBytesWritten;
    bWriteRC = WriteFile(hCom, &txchar, 1, &iBytesWritten,NULL);
    return;
}
//-----------------------------------------------------------------------------
void rs232_putstr(const char *string)
{
#ifdef DEBUG
    printf("%s\n", string);
#endif
    while (*string != '\0')
    {
        //delay_ms(5);          //Not Needed
        rs232_putch(*string++);
    }
}
//-----------------------------------------------------------------------------
float map(float in_x, float in_min, float in_max, float out_min, float out_max)
{
 return (in_x - in_min)*(out_max - out_min + 1)/(in_max - in_min + 1)+out_min;
}

